{ open Parser }

let digit = ['0' - '9']
let digits = digit+

rule tokenize = parse
  [' ' '\t' '\r' '\n'] { tokenize lexbuf }
| "/*" { comment lexbuf }
(* Operations *)
| '+' { PLUS }
| '-' { MINUS }
| '*' { TIMES }
| '/' { DIVIDE }
| '%' { MOD }
(* Braces/Brakets/Paren etc *)
| '(' { LPAREN }
| ')' { RPAREN }
| '[' { LBRACK }
| ']' { RBRACK }
| ';' { SEMIC }
| ',' { COMMA }
| '{' { LBRACE }
| '}' { RBRACE }
| '=' { ASSIGN }
| '.' { DOT }
(* Keywords *)
| "def" { DEF }
| "return" { RETURN }
| "while" { WHILE }
| "for" { FOR }
| "new" { NEW }
| "rhythm" { RHYTHM }
| "pitch"  { PITCH }
| "length" { LENGTH }
(* Types *)
| "void" { VOID }
| "bool" { BOOL }
| "int" { INT }
| "Note" { NOTE }
| "float" { FLOAT }
| "String" { STRING }
(* Loops/Statements *)
| "if"     { IF }
| "else"   { ELSE }
(* Comparisons *)
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "&&"     { AND }
| "||"     { OR }
(* Unary Operators *)
| "!" {NOT}
(* | ">>" {SEMUP}
| "<<" {SEMDOWN}
| "++" {OCTUP}
| "--" {OCTDOWN} *)
(* Literals *)
| "true"   { BLIT(true)  }
| "false"  { BLIT(false) }
| ['A'-'G']['@' '#']?digit as pitch { PLIT(pitch) }
| 't''s' | 's''t' | 'e''t' | 'q''r' | 'h''f' | 'w''h' as lit {RLIT(lit)}
| ['a'-'z']['a'-'z' 'A'-'Z' '0'-'9' '_']*  as id { ID(id) }
| (digits '.' digit*) | ('.' digits) as lxm { FLIT(lxm) }
| '"' {str (Buffer.create 16) lexbuf} 
| digits as lxm { LITERAL(int_of_string lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }


and comment = parse
  "*/" { tokenize lexbuf }
 | _ { comment lexbuf }

and str buf = parse 
'"' { STRLITERAL( Buffer.contents buf) }
| _ {Buffer.add_string buf (Lexing.lexeme lexbuf); str buf lexbuf}


